import Foundation
import Phidget22_C

/**
The core Phidget class deals with functionality common to all Phidgets, such as opening and closing them, or setting Attach, Detach and Event handlers.

This class also is used to specity the associations between the Phidget software objects and their corresponding physical devices, and makes it possible to determine which Phidget is which in cases where it might otherwise be ambiguous.
*/
public class PhidgetBase {

	var chandle: PhidgetHandle?
	var retained: Bool = false
	var selfCtx: Unmanaged<PhidgetBase>?

	init(_ handle: PhidgetHandle) {
		chandle = handle
		selfCtx = Unmanaged.passUnretained(self)
	}

	deinit {
		if (retained) {
			Phidget_release(&chandle)
		}
	}

	/**
	Pass to `setDeviceSerialNumber()` to open any serial number.
	*/
	public static let anySerialNumber: Int = -1
	/**
	Pass to `setHubPort()` to open any hub port.
	*/
	public static let anyHubPort: Int = -1
	/**
	Pass to `setChannel()` to open any channel.
	*/
	public static let anyChannel: Int = -1
	/**
	Pass to `setDeviceLabel()` to open any label.
	*/
	public static let anyLabel: String? = nil
	/**
	Pass to `open()` for an infinite timeout.
	*/
	public static let infiniteTimeout: UInt32 = 0
	/**
	Pass to `open()` for the default timeout.
	*/
	public static let defaultTimeout: UInt32 = 500

	/**
	Release memory and threads used by the Phidget library. Should be called prior to unloading the library from the address space.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- flags: Reserved for future use. Pass 0.
	*/
	public static func finalize(flags: Int = 0) throws {
		let result: PhidgetReturnCode
		result = Phidget_finalize(Int32(flags))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	The Phidget library version.

	- returns:
	The library version

	- throws:
	An error or type `PhidgetError`
	*/
	public static func getLibraryVersion() throws -> String {
		let result: PhidgetReturnCode
		var libraryVersion: UnsafePointer<CChar>?
		result = Phidget_getLibraryVersion(&libraryVersion)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: libraryVersion!)
	}

	/**
	Gets the attached status of channel.

	*   Most API calls are only valid on attached Phidgets.

	- returns:
	True if the channel is attached

	- throws:
	An error or type `PhidgetError`
	*/
	public func getAttached() throws -> Bool {
		let result: PhidgetReturnCode
		var attached: Int32 = 0
		result = Phidget_getAttached(chandle, &attached)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return (attached == 0 ? false : true)
	}

	/**
	Gets the channel index of this channel.

	- returns:
	The channel index

	- throws:
	An error or type `PhidgetError`
	*/
	public func getChannel() throws -> Int {
		let result: PhidgetReturnCode
		var channel: Int32 = 0
		result = Phidget_getChannel(chandle, &channel)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return Int(channel)
	}

	/**
	Specifies the channel index to be opened. The default channel is 0.

	`anyChannel` may be specified to indicate any channel that matches the device class.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- channel: The channel index
	*/
	public func setChannel(_ channel: Int) throws {
		let result: PhidgetReturnCode
		result = Phidget_setChannel(chandle, Int32(channel))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the class for channel.

	- returns:
	The channel class

	- throws:
	An error or type `PhidgetError`
	*/
	public func getChannelClass() throws -> ChannelClass {
		let result: PhidgetReturnCode
		var channelClass: Phidget_ChannelClass = PHIDCHCLASS_NOTHING
		result = Phidget_getChannelClass(chandle, &channelClass)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return ChannelClass(rawValue: channelClass.rawValue)!
	}

	/**
	Gets the class name for this channel.

	- returns:
	The channels class name

	- throws:
	An error or type `PhidgetError`
	*/
	public func getChannelClassName() throws -> String {
		let result: PhidgetReturnCode
		var channelClassName: UnsafePointer<CChar>?
		result = Phidget_getChannelClassName(chandle, &channelClassName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: channelClassName!)
	}

	/**
	Gets the name for this channel.

	- returns:
	The channels name

	- throws:
	An error or type `PhidgetError`
	*/
	public func getChannelName() throws -> String {
		let result: PhidgetReturnCode
		var channelName: UnsafePointer<CChar>?
		result = Phidget_getChannelName(chandle, &channelName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: channelName!)
	}

	/**
	Gets the sub class for this channel.

	- returns:
	The channels subclass

	- throws:
	An error or type `PhidgetError`
	*/
	public func getChannelSubclass() throws -> ChannelSubclass {
		let result: PhidgetReturnCode
		var channelSubclass: Phidget_ChannelSubclass = PHIDCHSUBCLASS_NONE
		result = Phidget_getChannelSubclass(chandle, &channelSubclass)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return ChannelSubclass(rawValue: channelSubclass.rawValue)!
	}

	/**
	Closes a Phidget channel that has been opened.`close()` will release the channel on the Phidget device, and should be called prior to delete.

	- throws:
	An error or type `PhidgetError`
	*/
	public func close() throws {
		let result: PhidgetReturnCode
		result = Phidget_close(chandle)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the number of channels of the specified channel type on the device. Pass PHIDCHCLASS_NOTHING to get the total number of channels.

	- returns:
	The Channel Count

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- cls: The Channel Class
	*/
	public func getDeviceChannelCount(cls: ChannelClass) throws -> UInt32 {
		let result: PhidgetReturnCode
		var count: UInt32 = 0
		result = Phidget_getDeviceChannelCount(chandle, Phidget_ChannelClass(cls.rawValue), &count)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return count
	}

	/**
	Gets the device class for the Phidget which this channel is a part of.

	- returns:
	The class of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceClass() throws -> DeviceClass {
		let result: PhidgetReturnCode
		var deviceClass: Phidget_DeviceClass = PHIDCLASS_NOTHING
		result = Phidget_getDeviceClass(chandle, &deviceClass)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return DeviceClass(rawValue: deviceClass.rawValue)!
	}

	/**
	Gets the device class name for the Phidget which this channel is a part of.

	- returns:
	the class name of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceClassName() throws -> String {
		let result: PhidgetReturnCode
		var deviceClassName: UnsafePointer<CChar>?
		result = Phidget_getDeviceClassName(chandle, &deviceClassName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: deviceClassName!)
	}

	/**
	Gets the device id for the Phidget which this channel is a part of.

	- returns:
	The id of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceID() throws -> DeviceID {
		let result: PhidgetReturnCode
		var deviceID: Phidget_DeviceID = PHIDID_NOTHING
		result = Phidget_getDeviceID(chandle, &deviceID)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return DeviceID(rawValue: deviceID.rawValue)!
	}

	/**
	Gets the label of the Phidget which this channel is a part of.

	- returns:
	The device label

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceLabel() throws -> String {
		let result: PhidgetReturnCode
		var deviceLabel: UnsafePointer<CChar>?
		result = Phidget_getDeviceLabel(chandle, &deviceLabel)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: deviceLabel!)
	}

	/**
	Specifies the label of the Phidget to be opened. Leave un-set to open any label.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- deviceLabel: The device label
	*/
	public func setDeviceLabel(_ deviceLabel: String?) throws {
		let result: PhidgetReturnCode
		result = Phidget_setDeviceLabel(chandle, deviceLabel)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the name of the Phidget which this channel is a part of.

	- returns:
	The name of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceName() throws -> String {
		let result: PhidgetReturnCode
		var deviceName: UnsafePointer<CChar>?
		result = Phidget_getDeviceName(chandle, &deviceName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: deviceName!)
	}

	/**
	Gets the serial number of the Phidget which this channel is a part of.

	- returns:
	The device serial number

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceSerialNumber() throws -> Int {
		let result: PhidgetReturnCode
		var deviceSerialNumber: Int32 = 0
		result = Phidget_getDeviceSerialNumber(chandle, &deviceSerialNumber)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return Int(deviceSerialNumber)
	}

	/**
	Specifies the serial number of the Phidget to be opened. Leave un-set to open any serial number.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- deviceSerialNumber: The device serial number
	*/
	public func setDeviceSerialNumber(_ deviceSerialNumber: Int) throws {
		let result: PhidgetReturnCode
		result = Phidget_setDeviceSerialNumber(chandle, Int32(deviceSerialNumber))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the SKU (part number) of the Phidget which this channel is a part of.

	- returns:
	the SKU of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceSKU() throws -> String {
		let result: PhidgetReturnCode
		var deviceSKU: UnsafePointer<CChar>?
		result = Phidget_getDeviceSKU(chandle, &deviceSKU)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: deviceSKU!)
	}

	/**
	Gets the version of the Phidget which this channel is a part of.

	- returns:
	The version of the channels device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getDeviceVersion() throws -> Int {
		let result: PhidgetReturnCode
		var deviceVersion: Int32 = 0
		result = Phidget_getDeviceVersion(chandle, &deviceVersion)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return Int(deviceVersion)
	}

	/**
	Gets the hub that this channel is attached to.

	- returns:
	The hub the channels device is attached to

	- throws:
	An error or type `PhidgetError`
	*/
	public func getHub() throws -> Phidget? {
		let result: PhidgetReturnCode
		var hub: PhidgetHandle?
		result = Phidget_getHub(chandle, &hub)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		if hub == nil { return nil }
		return Manager.createRetainedTypedPhidget(hub!)
	}

	/**
	Gets the hub port index that this Phidget is attached to.

	- returns:
	The hub port index

	- throws:
	An error or type `PhidgetError`
	*/
	public func getHubPort() throws -> Int {
		let result: PhidgetReturnCode
		var hubPort: Int32 = 0
		result = Phidget_getHubPort(chandle, &hubPort)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return Int(hubPort)
	}

	/**
	Specifies the hub port index to open this channel on.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- hubPort: The hub port index
	*/
	public func setHubPort(_ hubPort: Int) throws {
		let result: PhidgetReturnCode
		result = Phidget_setHubPort(chandle, Int32(hubPort))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the number of VINT ports on the hub which this Phidget is attached to.

	- returns:
	The number of ports on the hub the channels device is attachd to

	- throws:
	An error or type `PhidgetError`
	*/
	public func getHubPortCount() throws -> Int {
		let result: PhidgetReturnCode
		var hubPortCount: Int32 = 0
		result = Phidget_getHubPortCount(chandle, &hubPortCount)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return Int(hubPortCount)
	}

	/**
	Returns true if the Phidget handle is for a channel.

	- returns:
	True if the handle is a channel

	- throws:
	An error or type `PhidgetError`
	*/
	public func getIsChannel() throws -> Bool {
		let result: PhidgetReturnCode
		var isChannel: Int32 = 0
		result = Phidget_getIsChannel(chandle, &isChannel)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return (isChannel == 0 ? false : true)
	}

	/**
	Whether this channel is a hub port channel, or part of a VINT device attached to a hub port.

	- returns:
	The hub port mode

	- throws:
	An error or type `PhidgetError`
	*/
	public func getIsHubPortDevice() throws -> Bool {
		let result: PhidgetReturnCode
		var isHubPortDevice: Int32 = 0
		result = Phidget_getIsHubPortDevice(chandle, &isHubPortDevice)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return (isHubPortDevice == 0 ? false : true)
	}

	/**
	Specifies whether this channel should be opened on a hub port directly, or on a VINT device attached to a hub port.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- isHubPortDevice: The hub port mode
	*/
	public func setIsHubPortDevice(_ isHubPortDevice: Bool) throws {
		let result: PhidgetReturnCode
		result = Phidget_setIsHubPortDevice(chandle, (isHubPortDevice ? 1 : 0))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Returns true when this channel is attached directly; otherwise, false.

	- returns:
	True if the channel is attached to a local device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getIsLocal() throws -> Bool {
		let result: PhidgetReturnCode
		var isLocal: Int32 = 0
		result = Phidget_getIsLocal(chandle, &isLocal)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return (isLocal == 0 ? false : true)
	}

	/**
	Specifies that this channel will be opened directly, or not.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- isLocal: True if the channel is attached to a local device
	*/
	public func setIsLocal(_ isLocal: Bool) throws {
		let result: PhidgetReturnCode
		result = Phidget_setIsLocal(chandle, (isLocal ? 1 : 0))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Returns true when this channel is attached via a Phidget network server; otherwise, false.

	- returns:
	True if the channel is attached to a network device

	- throws:
	An error or type `PhidgetError`
	*/
	public func getIsRemote() throws -> Bool {
		let result: PhidgetReturnCode
		var isRemote: Int32 = 0
		result = Phidget_getIsRemote(chandle, &isRemote)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return (isRemote == 0 ? false : true)
	}

	/**
	Specifies that this channel will be opened remotely, or not.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- isRemote: True if the channel is attached to a network device
	*/
	public func setIsRemote(_ isRemote: Bool) throws {
		let result: PhidgetReturnCode
		result = Phidget_setIsRemote(chandle, (isRemote ? 1 : 0))
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Opens the Phidget channel.The specific channel to be opened can be specified by setting any of the following properties:

	*   DeviceSerialNumber
	*   DeviceLabel
	*   Channel
	*   ServerName
	*   IsLocal
	*   IsRemote
	*   HubPort
	*   HubSerialNumber
	*   HubLabel
	*   IsHubPortDevice

	Open will block until the channel is opened or a timeout occurs. A timeout value of 0 will wait forever.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- timeout: Timeout in milliseconds
	*/
	public func open(timeout: UInt32) throws {
		let result: PhidgetReturnCode
		result = Phidget_openWaitForAttachment(chandle, timeout)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Opens the Phidget channel. The specific channel to be opened can be specified by setting any of the following properties:

	*   DeviceSerialNumber
	*   DeviceLabel
	*   Channel
	*   ServerName
	*   IsLocal
	*   IsRemote
	*   HubPort
	*   HubSerialNumber
	*   HubLabel
	*   IsHubPortDevice

	Open will return immediately, with the open proceding asynchronously.Use the Attach event or Attached property to determine when the channel is ready to use.

	- throws:
	An error or type `PhidgetError`
	*/
	public func open() throws {
		let result: PhidgetReturnCode
		result = Phidget_open(chandle)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the handle of the parent of the given handle.

	*   May be used to travel up the device to the root device
	*   The root device will return a null handle
	*   Parent handles always refer to devices. See `IsChannel`

	- returns:
	The handle of the parent

	- throws:
	An error or type `PhidgetError`
	*/
	public func getParent() throws -> Phidget? {
		let result: PhidgetReturnCode
		var parent: PhidgetHandle?
		result = Phidget_getParent(chandle, &parent)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		if parent == nil { return nil }
		return Manager.createRetainedTypedPhidget(parent!)
	}

	/**
	Gets the hostname of the Phidget server for network attached Phidgets.

	- returns:
	The hostname of the channel's server

	- throws:
	An error or type `PhidgetError`
	*/
	public func getServerHostname() throws -> String {
		let result: PhidgetReturnCode
		var serverHostname: UnsafePointer<CChar>?
		result = Phidget_getServerHostname(chandle, &serverHostname)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: serverHostname!)
	}

	/**
	Gets the name of the Phidget network server for network attached Phidgets.

	- returns:
	The name of the server the channel is from

	- throws:
	An error or type `PhidgetError`
	*/
	public func getServerName() throws -> String {
		let result: PhidgetReturnCode
		var serverName: UnsafePointer<CChar>?
		result = Phidget_getServerName(chandle, &serverName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: serverName!)
	}

	/**
	Specifies that this channel will be opened remotely, on a Phidget server with this name.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- serverName: The name of the server the channel is from
	*/
	public func setServerName(_ serverName: String) throws {
		let result: PhidgetReturnCode
		result = Phidget_setServerName(chandle, serverName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	/**
	Gets the peer name (address and port) of the Phidget server for network attached Phidgets.

	- returns:
	The address and port of the channel's server

	- throws:
	An error or type `PhidgetError`
	*/
	public func getServerPeerName() throws -> String {
		let result: PhidgetReturnCode
		var serverPeerName: UnsafePointer<CChar>?
		result = Phidget_getServerPeerName(chandle, &serverPeerName)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
		return String(cString: serverPeerName!)
	}

	/**
	Writes a string to the device flash. This string will persist across power cycles.  
	The max String length is 10 character UTF-8. Some devices can not have their labels set from Windows. For these devices the label should be set from Linux or OS X.

	- throws:
	An error or type `PhidgetError`

	- parameters:
		- deviceLabel: The device label
	*/
	public func writeDeviceLabel(deviceLabel: String) throws {
		let result: PhidgetReturnCode
		result = Phidget_writeDeviceLabel(chandle, deviceLabel)
		if result != EPHIDGET_OK {
			throw (PhidgetError(code: result))
		}
	}

	internal func initializeEvents() { initializeBaseEvents() }
	internal func initializeBaseEvents() {
		Phidget_setOnAttachHandler(chandle, nativeAttachHandler, UnsafeMutableRawPointer(selfCtx!.toOpaque()))
		Phidget_setOnDetachHandler(chandle, nativeDetachHandler, UnsafeMutableRawPointer(selfCtx!.toOpaque()))
		Phidget_setOnErrorHandler(chandle, nativeErrorHandler, UnsafeMutableRawPointer(selfCtx!.toOpaque()))
		Phidget_setOnPropertyChangeHandler(chandle, nativePropertyChangeHandler, UnsafeMutableRawPointer(selfCtx!.toOpaque()))
	}

	internal func uninitializeEvents() { uninitializeBaseEvents() }
	internal func uninitializeBaseEvents() {
		Phidget_setOnAttachHandler(chandle, nil, nil)
		Phidget_setOnDetachHandler(chandle, nil, nil)
		Phidget_setOnErrorHandler(chandle, nil, nil)
		Phidget_setOnPropertyChangeHandler(chandle, nil, nil)
	}

	public let attach = SimpleEvent<Phidget> ()
	let nativeAttachHandler : Phidget_OnAttachCallback = { ch, ctx in
		let me = Unmanaged<Phidget>.fromOpaque(ctx!).takeUnretainedValue()
		me.attach.raise(me);
	}

	public let detach = SimpleEvent<Phidget> ()
	let nativeDetachHandler : Phidget_OnDetachCallback = { ch, ctx in
		let me = Unmanaged<Phidget>.fromOpaque(ctx!).takeUnretainedValue()
		me.detach.raise(me);
	}

	public let error = Event<Phidget, (code: ErrorEventCode, description: String)> ()
	let nativeErrorHandler : Phidget_OnErrorCallback = { ch, ctx, code, description in
		let me = Unmanaged<Phidget>.fromOpaque(ctx!).takeUnretainedValue()
		me.error.raise(me, (ErrorEventCode(rawValue: code.rawValue)!, String(cString: description!)));
	}

	public let propertyChange = Event<Phidget, String> ()
	let nativePropertyChangeHandler : Phidget_OnPropertyChangeCallback = { ch, ctx, propertyName in
		let me = Unmanaged<Phidget>.fromOpaque(ctx!).takeUnretainedValue()
		me.propertyChange.raise(me, String(cString: propertyName!));
	}

}
